@@ -28,8 +28,8 @@
 #' into prefixes, suffixes, and "words".
 #'
 #' @param word Word to tokenize.
-#' @param vocab_split List of named integer vector containing vocabulary words.
-#'   Should have components named "prefixes", "words", "suffixes"
+#' @param vocab_split List of character vectors containing vocabulary words.
+#'   Should have components named "prefixes", "words", "suffixes".
 #' @param dir Integer; if 1 (the default), look for tokens starting at the
 #'   beginning of the word. Otherwise, start at the end.
 #' @param allow_compounds Logical; whether to allow multiple whole words in the
@@ -100,22 +100,22 @@
       sub_str <- substr(word, start, end) # inclusive on both ends
 
       # first look for prefixes, if allowed
-      if ("p" %in% allowed_next & end < wordlen & sub_str %in% prefixes) {
+      if ("p" %fin% allowed_next & end < wordlen & sub_str %fin% prefixes) {
         cur_substr <- paste0(sub_str, frag_pat)
         allowed_next <- allowed_next_rules[["p"]]
         break
       }
       # next, look for suffix-like pieces, if we're not at start of word
-      if ("s" %in% allowed_next & start > 1 & sub_str %in% suffixes) {
+      if ("s" %fin% allowed_next & start > 1 & sub_str %fin% suffixes) {
         cur_substr <- paste0(frag_pat, sub_str)
         allowed_next <- allowed_next_rules[["s"]]
         break
       }
       # finally, look for complete words, if allowed
-      if (any(c("w", "#") %in% allowed_next) & sub_str %in% words) {
+      if (any(c("w", "#") %fin% allowed_next) & sub_str %fin% words) {
         cur_substr <- sub_str
         # insert the frag_pat token in, if we're between complete words
-        if ("#" %in% allowed_next) {
+        if ("#" %fin% allowed_next) {
           if (dir == 1) {
             cur_substr <- append(frag_pat, cur_substr)
           } else {
@@ -164,9 +164,8 @@
 #' pieces.
 #'
 #' @param word Character scalar; word to tokenize.
-#' @param vocab Named integer vector containing vocabulary words. Should have
-#'   "vocab_split" attribute, with components named "prefixes", "words",
-#'   "suffixes".
+#' @param vocab_split List of character vectors containing vocabulary words.
+#'   Should have components named "prefixes", "words", "suffixes".
 #' @param unk_token Token to represent unknown words.
 #' @param max_chars Maximum length of word recognized.
 #' @param allow_compounds Logical; whether to allow multiple whole words in the
@@ -176,11 +175,11 @@
 #' @return Input word as a list of tokens.
 #' @keywords internal
 .mp_tokenize_word_bidir <- function(word,
-                                    vocab,
+                                    vocab_split,
                                     unk_token,
                                     max_chars,
                                     allow_compounds = TRUE) {
-  vocab_split <- attr(vocab, "vocab_split")
+  # vocab_split <- attr(vocab, "vocab_split")
   t1 <- .mp_tokenize_word(word, vocab_split,
                           dir = 1,
                           allow_compounds = allow_compounds,
@@ -208,9 +207,9 @@
 
 #' Tokenize an Input Word-by-word
 #'
+#' @inheritParams .mp_tokenize_word_lookup
 #' @param words Character; a vector of words (generated by space-tokenizing a
 #'   single input).
-#' @inheritParams .mp_tokenize_word_lookup
 #'
 #' @return A named integer vector of tokenized words.
 #' @keywords internal
@@ -240,6 +239,7 @@
 #' Look up a word in the table; go to fall-back otherwise.
 #'
 #' @inheritParams .mp_tokenize_word_bidir
+#' @param vocab A morphemepiece vocabulary.
 #' @param lookup A morphemepiece lookup table.
 #'
 #' @return Input word, broken into tokens.
@@ -249,34 +249,50 @@
                                      lookup,
                                      unk_token,
                                      max_chars) {
+  # first get the split_vocab from the attributes...
+  vocab_split <- attr(vocab, "vocab_split")
+  # Now we just need the vocab itself, with no other attributes.
+  # Think about moving this higher up in the chain. But then we'd need to keep
+  # the split vocab around till now.
+  vocab <- .process_mp_vocab(vocab) 
+
   if (word == "") {
     return(integer(0))
   }
-  if (word %in% names(vocab)) { # punctuation, etc.
-    return(vocab[word])
+  if (word %fin% vocab) { # includes punctuation, etc.
+    # Get ID by position.
+    id <- fastmatch::fmatch(word, vocab)
+    names(id) <- word
+    return(id - 1L) # default to 0-based index, for historical consistency
   }
-  # may as well remove unbroken words from lookup, even though it's relatively
-  # small component?
-  # mean(purrr::map_int(lookup, grepl, pattern="##"))
-  if (word %in% names(lookup)) {
+
+  # for profiling purposes
+  nl <- names(lookup)
+  if (word %fin% nl) {
     breakdown <- lookup[[word]]
     token_list <- stringr::str_split(breakdown, pattern = " ")[[1]]
   } else {
-    token_list <- .mp_tokenize_word_bidir(word, vocab, unk_token, max_chars)
+    token_list <- .mp_tokenize_word_bidir(word,
+                                          vocab_split,
+                                          unk_token,
+                                          max_chars)
   }
-  return(vocab[token_list])
+
+  # Get IDs by position.
+  ids <- fastmatch::fmatch(token_list, vocab)
+  names(ids) <- token_list
+  return(ids - 1L) # default to 0-based index, for historical consistency
 }
 
 
 # morphemepiece_tokenize --------------------------------------------------
 
-
 #' Tokenize Sequence with Morpheme Pieces
 #'
 #' Given a single sequence of text and a morphemepiece vocabulary, tokenizes the
 #' text.
 #'
-#' @inheritParams .mp_tokenize_word_lookup
+#' @inheritParams .mp_tokenize_single_string
 #' @param text Character scalar; text to tokenize.
 #'
 #' @return A character vector of tokenized text (later, this should be a named
@@ -307,6 +323,8 @@ morphemepiece_tokenize <- function(text,
     max_chars = max_chars
   )
   # Length-0 token vectors need to still be named.
-  tokens[lengths(tokens) == 0] <- list(vocab[character(0)])
+  empty_int <- integer(0)
+  names(empty_int) <- character(0)
+  tokens[lengths(tokens) == 0] <- list(empty_int)
   return(tokens)
 }
